"""Utility code for working with Concrete
"""

from concrete_uuid import *  # noqa
from file_io import *  # noqa
from mem_io import *  # noqa
from redis_io import *  # noqa
from json_fu import *  # noqa
from references import *  # noqa
from tokenization import *  # noqa
from unnone import *  # noqa



import logging

from concrete.structure.ttypes import TokenizationKind


def get_tokens(tokenization, suppress_warnings=False):
    '''
    Return list of Tokens from lattice.cachedBestPath, if Tokenization
    kind is TOKEN_LATTICE; else, return list of Tokens from tokenList.

    Warn and return list of Tokens from tokenList if kind is not set.
    Return None if kind is set but the respective data fields are not.
    '''

    if tokenization.kind is None:
        if not suppress_warnings:
            logging.warn('Tokenization.kind is None but is required in '
                         'Concrete; using tokenList')

        token_list = tokenization.tokenList
        if token_list.tokenList is not None:
            return token_list.tokenList

    elif tokenization.kind == TokenizationKind.TOKEN_LATTICE:
        token_lattice = tokenization.lattice
        if token_lattice.cachedBestPath is not None:
            lattice_path = token_lattice.cachedBestPath
            if lattice_path.tokenList is not None:
                return lattice_path.tokenList

    elif tokenization.kind == TokenizationKind.TOKEN_LIST:
        token_list = tokenization.tokenList
        if token_list.tokenList is not None:
            return token_list.tokenList

    else:
        raise ValueError('Unrecognized TokenizationKind %s'
                         % tokenization.kind)

    return None


def get_tagged_tokens(tokenization, tagging_type):
    '''
    Return list of TaggedTokens of taggingType equal to tagging_type,
    if there is a unique choice.

    Raise exception if there is no matching tagging or more than one
    matching tagging.
    '''
    tts = [
        tt
        for tt in tokenization.tokenTaggingList
        if tt.taggingType == tagging_type
    ]
    if len(tts) == 0:
        raise Exception('No %s tagging.' % tagging_type)
    elif len(tts) == 1:
        return tts[0].taggedTokenList
    else:
        raise Exception('More than one %s tagging.' % tagging_type)


def get_lemmas(t):
    return get_tagged_tokens(t, 'LEMMA')


def get_pos(t):
    return get_tagged_tokens(t, 'POS')


def plus(x, y):
    return x + y


def flatten(a):
    return reduce(plus, a, [])


def get_comm_tokens(comm, sect_pred=None, suppress_warnings=False):
    '''
    Return list of Tokens in communication, delegating to get_tokens
    for each sentence.
    '''
    return flatten(map(
        lambda sect: flatten(map(
            lambda sent: get_tokens(sent.tokenization, suppress_warnings),
            sect.sentenceList
        )),
        filter(sect_pred, comm.sectionList)
        if (sect_pred is not None) else comm.sectionList
    ))



# -*- coding: utf-8 -*-
#
# Autogenerated by Thrift Compiler (0.9.3)
#
# DO NOT EDIT UNLESS YOU ARE SURE THAT YOU KNOW WHAT YOU ARE DOING
#
#  options string: py:new_style,utf8strings,coding=utf-8
#

from thrift.Thrift import TType, TMessageType, TException, TApplicationException
import logging
from ttypes import *
from thrift.Thrift import TProcessor
from thrift.transport import TTransport
from thrift.protocol import TBinaryProtocol, TProtocol
try:
  from thrift.protocol import fastbinary
except:
  fastbinary = None


class Iface(object):
  """
  A service that exists so that clients can send Concrete data
  structures to implementing servers.

  Implement this if you are creating an analytic that wishes to
  send its results back to a server. That server may perform
  validation, write the new layers to a database, and so forth.
  """
  def send(self, communication):
    """
    Send a communication to a server implementing this method.

    The communication that is sent back should contain the new
    analytic layers you wish to append. You may also wish to call
    methods that unset annotations you feel the receiver would not
    find useful in order to reduce network overhead.

    Parameters:
     - communication
    """
    pass


class Client(Iface):
  """
  A service that exists so that clients can send Concrete data
  structures to implementing servers.

  Implement this if you are creating an analytic that wishes to
  send its results back to a server. That server may perform
  validation, write the new layers to a database, and so forth.
  """
  def __init__(self, iprot, oprot=None):
    self._iprot = self._oprot = iprot
    if oprot is not None:
      self._oprot = oprot
    self._seqid = 0

  def send(self, communication):
    """
    Send a communication to a server implementing this method.

    The communication that is sent back should contain the new
    analytic layers you wish to append. You may also wish to call
    methods that unset annotations you feel the receiver would not
    find useful in order to reduce network overhead.

    Parameters:
     - communication
    """
    self.send_send(communication)

  def send_send(self, communication):
    self._oprot.writeMessageBegin('send', TMessageType.ONEWAY, self._seqid)
    args = send_args()
    args.communication = communication
    args.write(self._oprot)
    self._oprot.writeMessageEnd()
    self._oprot.trans.flush()

class Processor(Iface, TProcessor):
  def __init__(self, handler):
    self._handler = handler
    self._processMap = {}
    self._processMap["send"] = Processor.process_send

  def process(self, iprot, oprot):
    (name, type, seqid) = iprot.readMessageBegin()
    if name not in self._processMap:
      iprot.skip(TType.STRUCT)
      iprot.readMessageEnd()
      x = TApplicationException(TApplicationException.UNKNOWN_METHOD, 'Unknown function %s' % (name))
      oprot.writeMessageBegin(name, TMessageType.EXCEPTION, seqid)
      x.write(oprot)
      oprot.writeMessageEnd()
      oprot.trans.flush()
      return
    else:
      self._processMap[name](self, seqid, iprot, oprot)
    return True

  def process_send(self, seqid, iprot, oprot):
    args = send_args()
    args.read(iprot)
    iprot.readMessageEnd()
    try:
      self._handler.send(args.communication)
      msg_type = TMessageType.REPLY
    except (TTransport.TTransportException, KeyboardInterrupt, SystemExit):
      raise
    except:
      pass


# HELPER FUNCTIONS AND STRUCTURES

class send_args(object):
  """
  Attributes:
   - communication
  """

  thrift_spec = (
    None, # 0
    (1, TType.STRUCT, 'communication', (concrete.communication.ttypes.Communication, concrete.communication.ttypes.Communication.thrift_spec), None, ), # 1
  )

  def __init__(self, communication=None,):
    self.communication = communication

  def read(self, iprot):
    if iprot.__class__ == TBinaryProtocol.TBinaryProtocolAccelerated and isinstance(iprot.trans, TTransport.CReadableTransport) and self.thrift_spec is not None and fastbinary is not None:
      fastbinary.decode_binary(self, iprot.trans, (self.__class__, self.thrift_spec))
      return
    iprot.readStructBegin()
    while True:
      (fname, ftype, fid) = iprot.readFieldBegin()
      if ftype == TType.STOP:
        break
      if fid == 1:
        if ftype == TType.STRUCT:
          self.communication = concrete.communication.ttypes.Communication()
          self.communication.read(iprot)
        else:
          iprot.skip(ftype)
      else:
        iprot.skip(ftype)
      iprot.readFieldEnd()
    iprot.readStructEnd()

  def write(self, oprot):
    if oprot.__class__ == TBinaryProtocol.TBinaryProtocolAccelerated and self.thrift_spec is not None and fastbinary is not None:
      oprot.trans.write(fastbinary.encode_binary(self, (self.__class__, self.thrift_spec)))
      return
    oprot.writeStructBegin('send_args')
    if self.communication is not None:
      oprot.writeFieldBegin('communication', TType.STRUCT, 1)
      self.communication.write(oprot)
      oprot.writeFieldEnd()
    oprot.writeFieldStop()
    oprot.writeStructEnd()

  def validate(self):
    return


  def __hash__(self):
    value = 17
    value = (value * 31) ^ hash(self.communication)
    return value

  def __repr__(self):
    L = ['%s=%r' % (key, value)
      for key, value in self.__dict__.iteritems()]
    return '%s(%s)' % (self.__class__.__name__, ', '.join(L))

  def __eq__(self, other):
    return isinstance(other, self.__class__) and self.__dict__ == other.__dict__

  def __ne__(self, other):
    return not (self == other)



from setuptools import setup
import glob

execfile('concrete/version.py')

setup(
    name="concrete",
    version=__version__,
    description="Python modules and scripts for working with Concrete",

    packages=[
        'concrete',

        # Python code generated by Thrift Compiler
        'concrete.audio',
        'concrete.clustering',
        'concrete.communication',
        'concrete.email',
        'concrete.entities',
        'concrete.exceptions',
        'concrete.language',
        'concrete.linking',
        'concrete.metadata',
        'concrete.nitf',
        'concrete.services',
        'concrete.situations',
        'concrete.spans',
        'concrete.structure',
        'concrete.twitter',
        'concrete.uuid',

        # Python code generated by people
        'concrete.util',
        ],

    scripts=glob.glob('scripts/*')
              + glob.glob('concrete/services/*-remote'),

    setup_requires=['pytest-runner'],
    tests_require=['pytest', 'pytest-mock', 'pytest-cov', 'flake8'],

    install_requires=[
        'networkx',
        'testfixtures',
        'thrift>=0.9.2',
        'redis>=2.10.0',
        'pycountry>=1.20'
        ],

    url="https://github.com/hltcoe/concrete-python",
    license="BSD",
)
